/*
	+---------------------------------------------------------------------+
	| SOR | TYPE | LEN | DATA | EOR | ... | SOR | TYPE | LEN | DATA | EOR |
	+---------------------------------------------------------------------+
*/

#include "ipc.h"

#define SAFEDELETE(x) {if(x != NULL) free(x); x = NULL;}

//write data to a shared memory buffer
BOOL IPCWrite(LPVOID lpMemAddr, PIPC_DATA pIpcData)
{
	PBYTE pMem = NULL, pEnd = NULL;
	DWORD i = 0, dwRecLen, dwIPCLen;	

	if (!pIpcData)
		return TRUE;	

	//size of structure
	dwIPCLen = sizeof(IPC_DATA) - sizeof(pIpcData->Buffer);

	pMem = (PBYTE)lpMemAddr;
	pEnd = ((PBYTE)lpMemAddr + IPC_BUFFER_LEN);

	//get the address of the first free record in ipc buffer
	while (pMem < pEnd)
	{
		if (*pMem == PAD)
		{ 
			pMem += 1;
			continue;
		}

		if (*pMem == SOR)
		{
			//read the length of the record
			pMem += 1;			
			i =  ((DWORD)*pMem) + dwIPCLen - 2;
			pMem += i;

			//check EOR
			if (*pMem == EOR)
			{
				pMem += 1;
				continue;
			}						

			//verify if the new record can fit the buffer
			if ((pMem + 1) >= pEnd)
			{
				pMem = (PBYTE)lpMemAddr;
				break;
			}			

			//verify if there's enough room for the new record
			if ((pMem + pIpcData->Length + dwIPCLen) >= pEnd)
			{
				pMem = (PBYTE)lpMemAddr;				
			}

			break;
		}
		else
			break;
	}

	pIpcData->StartOfRecord = SOR;
	pIpcData->EndOfRecord   = EOR;

	DWORD dwSize =	sizeof(pIpcData->StartOfRecord) + 
					sizeof(pIpcData->Length)		+
					sizeof(pIpcData->Type)			+ 					
					sizeof(pIpcData->dwTimestamp);

	//copy the start of record, type and the size tag
	memcpy(pMem, pIpcData, dwSize);
	pMem += dwSize;

	//copy the buffer
	memcpy(pMem, pIpcData->Buffer, pIpcData->Length);
	pMem += pIpcData->Length;

	//copy the end of record
	memcpy(pMem, &pIpcData->EndOfRecord, sizeof(pIpcData->EndOfRecord));

	return TRUE;
}

//free the ipc structure
BOOL FreeIPC(PIPC_DATA pIpc)
{
	if (!pIpc)
		return TRUE;

	SAFEDELETE(pIpc->Buffer);
	SAFEDELETE(pIpc);

	return TRUE;
}